@@ -1,9 +1,9 @@
 package App::wordlist;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-05-22'; # DATE
+our $DATE = '2020-05-24'; # DATE
 our $DIST = 'App-wordlist'; # DIST
-our $VERSION = '0.273'; # VERSION
+our $VERSION = '0.274'; # VERSION
 
 use 5.010001;
 use strict;
@@ -252,6 +252,7 @@ _
 };
 sub wordlist {
     require Encode;
+    require WordListUtil::CLI;
 
     my %args = @_;
 
@@ -301,7 +302,7 @@ sub wordlist {
             return [200] unless @$wordlists;
             my %all_stats;
             for my $wl (@$wordlists) {
-                my $mod = "WordList::$wl";
+                my $mod = "WordList::$wl"; $mod =~ s/=.*//;
                 (my $modpm = "$mod.pm") =~ s!::!/!g;
                 require $modpm;
                 if (@$wordlists == 1) {
@@ -314,24 +315,9 @@ sub wordlist {
         }
 
         # optimize random picking when there's only one wordlist to pick from
-        if ($random && @$wordlists == 1) {
-            no strict 'refs';
-            my $mod = "WordList::$wordlists->[0]";
-            (my $modpm = "$mod.pm") =~ s!::!/!g;
-            require $modpm;
-            my $dynamic = !${"$mod\::DYNAMIC"};
-
-            # not really faster than using each_word()
-            #if (!$dynamic && !$num) {
-            #    my $wl = $mod->new;
-            #    my $fh = \*{"$mod\::DATA"};
-            #    return [200, "OK", [shuffle <$fh>]];
-            #}
-
-            if ($num > 0 && $num <= 100) {
-                my $wl = $mod->new;
-                return [200, "OK", [$wl->pick($num)]];
-            }
+        if ($random && @$wordlists == 1 && $num > 0 && $num <= 100) {
+            my $wl_obj = WordListUtil::CLI::instantiate_wordlist($wordlists->[0]);
+            return [200, "OK", [$wl_obj->pick($num)]];
         }
 
         my $n = 0;
@@ -354,27 +340,22 @@ sub wordlist {
         };
 
         my $i_wordlist = 0;
+        my $nth_word;
         my $wl_obj;
         my $code_return_word = sub {
           REDO:
             return if $i_wordlist >= @$wordlists;
             my $wl = $wordlists->[$i_wordlist];
             unless ($wl_obj) {
-                my $mod = "WordList::$wl";
-                (my $modpm = "$mod.pm") =~ s!::!/!g;
-                log_trace "Loading wordlist module $mod ...";
-                eval {
-                    require $modpm;
-                    $wl_obj = $mod->new;
-                };
-                if ($@) {
-                    warn;
+                log_trace "Instantiating wordlist $wl ...";
+                $wl_obj = WordListUtil::CLI::instantiate_wordlist($wl, 'ignore');
+                unless ($wl_obj) {
                     $i_wordlist++;
                     goto REDO;
                 }
-                $wl_obj->reset_iterator;
+                $nth_word = 0;
             }
-            my $word = $wl_obj->next_word;
+            my $word = $nth_word++ ? $wl_obj->next_word : $wl_obj->first_word;
             unless (defined $word) {
                 undef $wl_obj;
                 $i_wordlist++;
@@ -513,7 +494,7 @@ App::wordlist - Grep words from WordList::*
 
 =head1 VERSION
 
-This document describes version 0.273 of App::wordlist (from Perl distribution App-wordlist), released on 2020-05-22.
+This document describes version 0.274 of App::wordlist (from Perl distribution App-wordlist), released on 2020-05-24.
 
 =head1 SYNOPSIS
 
@@ -653,13 +634,6 @@ that contains extra information.
 
 Return value:  (any)
 
-=head1 FAQ
-
-=head2 How to make wordlist return words in random order?
-
-The C<--random> (C<-r>) option was removed in v0.268. To return random words,
-you can pipe the output of C<wordlist> to C<shuf> or other similar utility.
-
 =head1 ENVIRONMENT
 
 =head2 DEBUG => bool
